@@ -3,7 +3,7 @@ package Net::DNS::Nameserver;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: Nameserver.pm 1813 2020-10-08 21:58:40Z willem $)[2];
+our $VERSION = (qw$Id: Nameserver.pm 1895 2023-01-16 13:38:08Z willem $)[2];
 
 
 =head1 NAME
@@ -21,7 +21,7 @@ Net::DNS::Nameserver - DNS server class
 
     my $nameserver = Net::DNS::Nameserver->new(
 	LocalAddr	=> '10.1.2.3',
-	LocalPort	=> 53,
+	LocalPort	=> 5353,
 	ReplyHandler	=> \&reply_handler
     );
 
@@ -39,7 +39,7 @@ See L</EXAMPLE> for an example.
 
 =cut
 
-use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1;'; ## no critic
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1;';    ## no critic
 require IO::Socket::INET unless USE_SOCKET_IP;
 
 use integer;
@@ -80,7 +80,7 @@ sub new {
 
 	# local server addresses must also be accepted by a resolver
 	my $LocalAddr = $self{LocalAddr} || [DEFAULT_ADDR];
-	my $resolver = Net::DNS::Resolver->new( nameservers => $LocalAddr );
+	my $resolver  = Net::DNS::Resolver->new( nameservers => $LocalAddr );
 	$resolver->force_v4(1) unless USE_SOCKET_IP;
 	$resolver->force_v4(1) if FORCE_IPv4;
 	my @localaddresses = $resolver->nameservers;
@@ -224,7 +224,8 @@ sub ReplyHandler {
 #------------------------------------------------------------------------------
 
 sub inet_new {
-	return USE_SOCKET_IP ? IO::Socket::IP->new(@_) : IO::Socket::INET->new(@_);
+	my @arg = @_;
+	return USE_SOCKET_IP ? IO::Socket::IP->new(@arg) : IO::Socket::INET->new(@arg);
 }
 
 #------------------------------------------------------------------------------
@@ -232,7 +233,7 @@ sub inet_new {
 #------------------------------------------------------------------------------
 
 sub make_reply {
-	my ( $self, $query, $peerhost, $conn ) = @_;
+	my ( $self, $query, $sock ) = @_;
 
 	unless ($query) {
 		print "ERROR: invalid packet\n" if $self->{Verbose};
@@ -271,8 +272,17 @@ sub make_reply {
 		my $id = $query->header->id;
 		print "query $id : $qname $qclass $qtype\n" if $self->{Verbose};
 
+		my $peer = $sock->peerhost;
+		my $conn = {
+			peerhost => $peer,
+			peerport => $sock->peerport,
+			protocol => $sock->protocol,
+			sockhost => $sock->sockhost,
+			sockport => $sock->sockport
+			};
+
 		my ( $rcode, $ans, $auth, $add );
-		my @arglist = ( $qname, $qclass, $qtype, $peerhost, $query, $conn );
+		my @arglist = ( $qname, $qclass, $qtype, $peer, $query, $conn );
 
 		if ( $opcode eq "QUERY" ) {
 			( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
@@ -406,13 +416,8 @@ sub tcp_connection {
 				print "Error decoding query packet: $err\n" if $self->{Verbose};
 				undef $query;			# force FORMERR reply
 			}
-			my $conn = {
-				sockhost => $sock->sockhost,
-				sockport => $sock->sockport,
-				peerhost => $sock->peerhost,
-				peerport => $sock->peerport
-				};
-			my $reply = $self->make_reply( $query, $sock->peerhost, $conn );
+
+			my $reply = $self->make_reply( $query, $sock );
 			if ( not defined $reply ) {
 				print "I couldn't create a reply for $peer. Closing socket.\n"
 						if $self->{Verbose};
@@ -452,7 +457,6 @@ sub udp_connection {
 		return;
 	}
 
-
 	print "UDP connection from $peerhost:$peerport to $sockhost\n" if $self->{Verbose};
 
 	my $query = Net::DNS::Packet->new( \$buf );
@@ -460,13 +464,8 @@ sub udp_connection {
 		print "Error decoding query packet: $err\n" if $self->{Verbose};
 		undef $query;					# force FORMERR reply
 	}
-	my $conn = {
-		sockhost => $sock->sockhost,
-		sockport => $sock->sockport,
-		peerhost => $sock->peerhost,
-		peerport => $sock->peerport
-		};
-	my $reply = $self->make_reply( $query, $peerhost, $conn ) || return;
+
+	my $reply = $self->make_reply( $query, $sock ) || return;
 
 	my $max_len = ( $query && $self->{Truncate} ) ? $query->edns->size : undef;
 	if ( $self->{Verbose} ) {
@@ -545,11 +544,11 @@ sub loop_once {
 
 			# If we have buffered output, then send as much as the OS will accept
 			# and wait with the rest
-			my $len = length $self->{_tcp}{$s}{outbuffer};
-			my $charssent = $sock->syswrite( $self->{_tcp}{$s}{outbuffer} ) || 0;
-			print "Sent $charssent of $len octets to ", $self->{_tcp}{$s}{peer}, ".\n"
+			my $len	 = length $self->{_tcp}{$s}{outbuffer};
+			my $sent = $sock->syswrite( $self->{_tcp}{$s}{outbuffer} ) || 0;
+			print "Sent $sent of $len octets to ", $self->{_tcp}{$s}{peer}, ".\n"
 					if $self->{Verbose};
-			substr( $self->{_tcp}{$s}{outbuffer}, 0, $charssent ) = "";
+			substr( $self->{_tcp}{$s}{outbuffer}, 0, $sent ) = "";
 			if ( length $self->{_tcp}{$s}{outbuffer} == 0 ) {
 				delete $self->{_tcp}{$s}{outbuffer};
 				$self->{_tcp}{$s}{state} = STATE_ACCEPTED;
@@ -632,7 +631,7 @@ could not be created.
 Each instance is configured using the following optional arguments:
 
     LocalAddr		IP address on which to listen	Defaults to loopback address
-    LocalPort		Port on which to listen		Defaults to 53
+    LocalPort		Port on which to listen		Defaults to 5353
     ZoneFile		Name of file containing RRs
 			accessed using the default
 			reply-handling subroutine
@@ -658,9 +657,8 @@ this may also include IPv6 addresses.
 
 
 The ReplyHandler subroutine is passed the query name, query class,
-query type and optionally an argument containing the peerhost, the
-incoming query, and the name of the incoming socket (sockethost). It
-must either return the response code and references to the answer,
+query type, peerhost, query record, and connection descriptor.
+It must either return the response code and references to the answer,
 authority, and additional sections of the response, or undef to leave
 the query unanswered.  Common response codes are:
 
@@ -685,10 +683,8 @@ See RFC 1035 and the IANA dns-parameters file for more information:
   ftp://ftp.rfc-editor.org/in-notes/rfc1035.txt
   http://www.isi.edu/in-notes/iana/assignments/dns-parameters
 
-The nameserver will listen for both UDP and TCP connections.  On
-Unix-like systems, the program will probably have to run as root
-to listen on the default port, 53.	A non-privileged user should
-be able to listen on ports 1024 and higher.
+The nameserver will listen for both UDP and TCP connections.
+On Unix-like systems, unprivileged users are denied access to ports below 1024.
 
 UDP reply truncation functionality was introduced in VERSION 830.
 The size limit is determined by the EDNS0 size advertised in the query,
@@ -800,9 +796,9 @@ additional filtering on its basis may be applied.
 
 =head1 BUGS
 
-Limitations in perl 5.8.6 makes it impossible to guarantee that
-replies to UDP queries from Net::DNS::Nameserver are sent from the
-IP-address they were received on. This is a problem for machines with
+Limitations in perl make it impossible to guarantee that replies to
+UDP queries from Net::DNS::Nameserver are sent from the IP-address
+to which the query was directed.  This is a problem for machines with
 multiple IP-addresses and causes violation of RFC2181 section 4.
 Thus a UDP socket created listening to INADDR_ANY (all available
 IP-addresses) will reply not necessarily with the source address being
@@ -835,7 +831,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -852,9 +848,10 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>,
-L<Net::DNS::Update>, L<Net::DNS::Header>, L<Net::DNS::Question>,
-L<Net::DNS::RR>, RFC 1035
+L<perl> L<Net::DNS> L<Net::DNS::Resolver> L<Net::DNS::Packet>
+L<Net::DNS::Update> L<Net::DNS::Header> L<Net::DNS::Question>
+L<Net::DNS::RR>
+L<RFC1035|https://tools.ietf.org/html/rfc1035>
 
 =cut
 
